/*
    GQ -- a GTK-based LDAP client
    Copyright (C) 1998,1999 Bert Vermeulen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <gtk/gtk.h>
#include <glib.h>
#include <gdk/gdkkeysyms.h>

#include <string.h>

#include "common.h"
#include "configfile.h"
#include "browse.h"
#include "mainwin.h"
#include "query.h"
#include "util.h"
#include "detail.h"
#include "debug.h"

extern GtkWidget *statusbar, *mainwin;
extern struct gq_config config;

GNode *root = NULL;
GtkWidget *pane1_scrwin = NULL;
GtkWidget *pane2_scrwin = NULL;
GtkWidget *browsemode_vbox = NULL;

GString *cur_dn = NULL;
struct ldapserver *cur_dn_server = NULL;

GtkWidget *create_mainwin_browsemode(void)
{
     GtkWidget *hbox1;
     GtkWidget *mainpane;
     GtkWidget *pane2_vbox;
     GtkWidget *treeroot;

     browsemode_vbox = gtk_vbox_new(FALSE, 0);
     hbox1 = gtk_hbox_new(FALSE, 0);
     gtk_widget_show(hbox1);
     gtk_box_pack_start(GTK_BOX(browsemode_vbox), hbox1, FALSE, FALSE, 3);

     mainpane = gtk_hpaned_new();
     gtk_container_border_width(GTK_CONTAINER(mainpane), 2);
     gtk_widget_show(mainpane);
     gtk_box_pack_start(GTK_BOX(browsemode_vbox), mainpane, TRUE, TRUE, 0);

     root = g_node_new(new_node_entry("root node"));

     treeroot = gtk_tree_new();
     gtk_signal_connect(GTK_OBJECT(treeroot), "select_child",
			GTK_SIGNAL_FUNC(server_item_selected), NULL);
     gtk_widget_show(treeroot);

     add_all_servers();
     gnode_to_tree(root, treeroot);

     pane1_scrwin = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pane1_scrwin),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
     gtk_widget_show(pane1_scrwin);
     gtk_widget_set_usize(pane1_scrwin, 300, -1);
     gtk_paned_add1(GTK_PANED(mainpane), pane1_scrwin);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pane1_scrwin),
					   treeroot);

     pane2_scrwin = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pane2_scrwin),
				    GTK_POLICY_AUTOMATIC,
				    GTK_POLICY_AUTOMATIC);
     gtk_widget_show(pane2_scrwin);
     pane2_vbox = gtk_vbox_new(FALSE, 5);
     gtk_widget_show(pane2_vbox);
     gtk_object_set_data(GTK_OBJECT(pane2_scrwin), "vbox", pane2_vbox);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pane2_scrwin),
					   pane2_vbox);
     gtk_paned_add2(GTK_PANED(mainpane), pane2_scrwin);

     gtk_widget_show(browsemode_vbox);
     gtk_object_set_data(GTK_OBJECT(browsemode_vbox), "focus", pane2_scrwin);

     cur_dn = g_string_sized_new(128);
     g_string_assign(cur_dn, "some dummy string");

     return(browsemode_vbox);
}


/*
 * add all configured servers to the root node
 */
void add_all_servers(void)
{
     GNode *new_node;
     struct ldapserver *server;
     struct node_entry *entry;
     int server_cnt;
     char message[128];

     server_cnt = 0;
     server = config.ldapservers;
     while(server) {
	  entry = new_node_entry(server->name);
	  entry->status = IS_SERVER;
	  new_node = g_node_new(entry);
	  g_node_append(root, new_node);
	  server = server->next;
	  server_cnt++;
     }

     make_message(message, server_cnt, "server", "servers", "found");
     statusbar_msg(message);

}


/*
 * build gtk_tree from GNode
 */
GtkWidget *gnode_to_tree(GNode *node, GtkWidget *tree)
{

     g_node_children_foreach(node, G_TRAVERSE_ALL,
			     (GNodeForeachFunc) gnode_to_tree_callback,
			     (gpointer) tree);

     return(tree);
}


gboolean gnode_to_tree_callback(GNode *node, gpointer tree)
{
     GtkWidget *new_item, *new_subtree;
     struct node_entry *entry;

     entry = (struct node_entry *) node->data;

     new_item = gtk_tree_item_new_with_label(dn_by_node(node));
     gtk_signal_connect(GTK_OBJECT(new_item), "button_press_event",
			GTK_SIGNAL_FUNC(button_press_on_tree_item), NULL);
     gtk_object_set_data(GTK_OBJECT(new_item), "node", (gpointer) node);
     gtk_widget_show(new_item);
     gtk_tree_append(GTK_TREE(tree), new_item);

     if(node->children) {
	  new_subtree = gtk_tree_new();
	  gtk_tree_item_set_subtree(GTK_TREE_ITEM(new_item), new_subtree);
	  gtk_signal_connect(GTK_OBJECT(new_subtree), "select_child",
			     GTK_SIGNAL_FUNC(tree_item_selected), NULL);
	  gnode_to_tree(node, new_subtree);
     }
     else {
	  switch(entry->status) {
	  case STATUS_UNKNOWN:
	       /* insert dummy subtree, connect expand to onelevel search */
	       new_subtree = gtk_tree_new();
	       gtk_tree_item_set_subtree(GTK_TREE_ITEM(new_item), new_subtree);
	       gtk_signal_connect(GTK_OBJECT(new_subtree), "select_child",
				  GTK_SIGNAL_FUNC(tree_item_selected), NULL);
	       gtk_signal_connect(GTK_OBJECT(new_item), "expand",
				  GTK_SIGNAL_FUNC(attach_children_to_node),
				  NULL);
	       break;

	  case IS_SERVER:
	       /* insert dummy subtree, connect expand to suffix scanner */
	       new_subtree = gtk_tree_new();
	       gtk_tree_item_set_subtree(GTK_TREE_ITEM(new_item), new_subtree);
	       gtk_signal_connect(GTK_OBJECT(new_subtree), "select_child",
				  GTK_SIGNAL_FUNC(tree_item_selected), NULL);
	       gtk_signal_connect(GTK_OBJECT(new_item), "expand",
				  GTK_SIGNAL_FUNC(attach_server_suffixes),
				  NULL);
	       break;

	  default:
	       break;
	  }
     }

     return(FALSE);
}


/*
 * remove server from visual and gnode trees
 */
void remove_server_node_tree(struct ldapserver *server)
{

     /* this will be called when a server was deleted with
	the preferences dialog */




}


/*
 * an item with a dummy subtree was selected or is about to be expanded. Find out
 * the child nodes for this DN, or (if there are none) delete the subtree
 */
void attach_children_to_node(GtkWidget *item)
{
     GNode *node;
     struct node_entry *entry;

     /* this function only gets called the first time an item's subtree
	is expanded -- should know whether it has children or not when done */
     gtk_signal_disconnect_by_func(GTK_OBJECT(item),
				   GTK_SIGNAL_FUNC(attach_children_to_node),
				   NULL);

     /* connect next expand sig */
     gtk_signal_connect(GTK_OBJECT(item), "expand",
			GTK_SIGNAL_FUNC(attach_children_to_tree), NULL);

     node = (GNode *) gtk_object_get_data(GTK_OBJECT(item), "node");

     set_busycursor();

     descend_onelevel(node);

     set_normalcursor();

     entry = (struct node_entry *) node->data;
     if(entry->status == HAS_CHILDREN) {
	  /* connect next expand sig */
	  gtk_signal_connect(GTK_OBJECT(item), "expand",
			     GTK_SIGNAL_FUNC(attach_children_to_tree), NULL);
     }
     else
	  gtk_tree_item_remove_subtree(GTK_TREE_ITEM(item));

}


/*
 * a subtree that's known to have children in it is about to be expanded.
 * attach_children_to_node() has already run, so just need to attach them to
 * the tree here
 */
void attach_children_to_tree(GtkWidget *item)
{
     GNode *node;
     GtkWidget *subtree;
     struct node_entry *entry;

     gtk_signal_disconnect_by_func(GTK_OBJECT(item),
				   GTK_SIGNAL_FUNC(attach_children_to_tree),
				   NULL);

     node = (GNode *) gtk_object_get_data(GTK_OBJECT(item), "node");

     entry = (struct node_entry *) node->data;
     if(entry->status == HAS_CHILDREN) {
	  subtree = GTK_TREE_ITEM_SUBTREE(GTK_TREE_ITEM(item));
	  gnode_to_tree(node, subtree);
     }

}


/*
 * a server with a dummy subtree is about to be expanded. Run the suffix
 * discovery stuff and attach any suffixes found. If none found, delete
 * the subtree. Which makes the server totally useless :-(
 */
void attach_server_suffixes(GtkWidget *item)
{
     GNode *server_node, *new_node;
     GSList *suffixes, *slist;
     GtkWidget *subtree;
     struct ldapserver *server;

     /* this function only gets called the first time a server is expanded */
     gtk_signal_disconnect_by_func(GTK_OBJECT(item),
				   GTK_SIGNAL_FUNC(attach_server_suffixes),
				   NULL);

     server_node = (GNode *) gtk_object_get_data(GTK_OBJECT(item), "node");
     server = server_by_node(server_node);

     suffixes = get_suffix(server);	 
     if( (slist = suffixes) ) {
	  /* add suffixes to server node */
	  while(slist) {
	       new_node = g_node_new(new_node_entry(slist->data));
	       g_node_append(server_node, new_node);
	       g_free(slist->data);
	       slist = slist->next;
	  }
	  g_slist_free(suffixes);

	  subtree = GTK_TREE_ITEM_SUBTREE(GTK_TREE_ITEM(item));
	  gnode_to_tree(server_node, subtree);
     }
     else
	  /* no suffixes found -- delete subtree */
	  gtk_tree_item_remove_subtree(GTK_TREE_ITEM(item));

}


/*
 * do a one level search of node's DN, and append any resulting entries
 * to the node
 */
void descend_onelevel(GNode *node)
{
     GNode *new_node;
     LDAP *ld;
     LDAPMessage *res, *e;
     struct ldapserver *server;
     struct node_entry *entry;
     int msg, rc, num_children;
     char *dn, message[MAX_DN_LEN + 21];
     char *dummy[] = { "dummy", NULL };

     entry = (struct node_entry *) node->data;
     server = server_by_node(node);
     if(!server)
	  return;

     if( (ld = open_connection(server)) == NULL)
	  return;

     sprintf(message, "onelevel search on %s", dn_by_node(node));
     statusbar_msg(message);

     if(config.sort_browse)
	  msg = ldap_search_s(ld, dn_by_node(node), LDAP_SCOPE_ONELEVEL,
			      "objectclass=*", dummy, 1, &res);
     else
	  msg = ldap_search(ld, dn_by_node(node), LDAP_SCOPE_ONELEVEL,
			    "objectclass=*", dummy, 1);

     if(msg == -1) {
	  statusbar_msg(ldap_err2string(msg));
	  return;
     }

     num_children = 0;

     if(config.sort_browse) {
	  if( (ldap_sort_entries(ld, &res, NULL, strcasecmp)) == LDAP_SUCCESS)
	       for(e = ldap_first_entry(ld, res); e != NULL; e=ldap_next_entry(ld, e)) {
		    dn = ldap_get_dn(ld, e);
		    new_node = g_node_new(new_node_entry(dn));
		    g_node_append(node, new_node);
		    free(dn);
		    num_children++;
	       }
     }
     else {
	  while( (rc = ldap_result(ld, msg, 0, NULL, &res)) == LDAP_RES_SEARCH_ENTRY)
	       for(e = ldap_first_entry(ld, res); e != NULL; e=ldap_next_entry(ld, e)) {
		    dn = ldap_get_dn(ld, e);
		    new_node = g_node_new(new_node_entry(dn));
		    g_node_append(node, new_node);
		    free(dn);
		    num_children++;
	       }
     }

     ldap_msgfree(res);

     make_message(message, num_children, "entry", "entries", "found");
     statusbar_msg(message);

     if(num_children)
	  entry->status = HAS_CHILDREN;
     else
	  entry->status = HAS_NO_CHILDREN;

}


/*
 * button pressed on a tree item. button 3 gets intercepted and puts up
 * a popup menu. All other buttons get passed along to the default handler
 */
gboolean button_press_on_tree_item(GtkWidget *tree_item, GdkEventButton *event)
{
     GNode *node;
     GtkWidget *root_menu, *menu, *menu_item;
     struct node_entry *entry;

     if(event->type == GDK_BUTTON_PRESS && event->button == 3) {
	  if( (node = (GNode *) gtk_object_get_data(GTK_OBJECT(tree_item), "node")) == NULL)
	       return(TRUE);

	  if( (entry = (struct node_entry *) node->data) == NULL)
	       return(TRUE);

	  root_menu = gtk_menu_item_new_with_label("Root");
	  gtk_widget_show(root_menu);
	  menu = gtk_menu_new();
	  gtk_menu_item_set_submenu(GTK_MENU_ITEM(root_menu), menu);

	  menu_item = gtk_tearoff_menu_item_new();
	  gtk_menu_append(GTK_MENU(menu), menu_item);
	  gtk_widget_set_sensitive(menu_item, FALSE);
	  gtk_widget_show(menu_item);

	  /* Refresh */
	  menu_item = gtk_menu_item_new_with_label("Refresh");
	  gtk_menu_append(GTK_MENU(menu), menu_item);
	  gtk_widget_show(menu_item);

	  if(entry->status == IS_SERVER)
	       gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
				  GTK_SIGNAL_FUNC(refresh_server),
				  (gpointer) tree_item);
	  else
	       gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
				  GTK_SIGNAL_FUNC(refresh_subtree),
				  (gpointer) tree_item);

	  gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
			 event->button, event->time);

	  gtk_signal_emit_stop_by_name(GTK_OBJECT(tree_item), "button_press_event");
     }

     return(TRUE);
}


void refresh_server(GtkWidget *widget, GtkWidget *tree_item)
{
     GNode *node;
     GList *items;
     GtkTree *tree;
     GtkWidget *new_subtree;
     struct ldapserver *server;
     int was_selected, was_expanded;

     if( (node = (GNode *) gtk_object_get_data(GTK_OBJECT(tree_item), "node")) == NULL)
	  return;

     if( (tree = GTK_TREE(GTK_TREE_ITEM(tree_item)->subtree)) == NULL)
	  return;

     if( (items = tree->children) == NULL)
	  return;

     was_selected = (GTK_WIDGET_STATE(tree_item) == GTK_STATE_SELECTED);
     was_expanded = GTK_TREE_ITEM(tree_item)->expanded;

     /* clear cached connection, if any */
     if( (server = server_by_node(node)) )
	  close_connection(server, TRUE);

     g_node_children_foreach(node, G_TRAVERSE_ALL,
			     (GNodeForeachFunc) node_entry_destroy, NULL);

     gtk_tree_remove_items(tree, items);

     new_subtree = gtk_tree_new();
     gtk_tree_item_set_subtree(GTK_TREE_ITEM(tree_item), new_subtree);
     gtk_signal_connect(GTK_OBJECT(new_subtree), "select_child",
			GTK_SIGNAL_FUNC(tree_item_selected), NULL);
     gtk_signal_connect(GTK_OBJECT(tree_item), "expand",
			GTK_SIGNAL_FUNC(attach_server_suffixes),
			NULL);

     if(was_expanded)
	  gtk_signal_emit_by_name(GTK_OBJECT(tree_item), "expand");

}


void refresh_subtree(GtkWidget *widget, GtkWidget *tree_item)
{
     GNode *node;
     GList *items;
     GtkTree *tree;
     GtkWidget *new_subtree;
     struct node_entry *entry;
     int was_selected, was_expanded;

     if( (node = (GNode *) gtk_object_get_data(GTK_OBJECT(tree_item), "node")) == NULL)
	  return;

     if( (entry = (struct node_entry *) node->data) == NULL)
	  return;

     was_selected = (GTK_WIDGET_STATE(tree_item) == GTK_STATE_SELECTED);
     was_expanded = GTK_TREE_ITEM(tree_item)->expanded;

     if( (GTK_TREE_ITEM(tree_item)->subtree) == NULL) {
	  /* item has no subtree at all (i.e. dummy subtree was removed)
	     so just refresh in right pane */
	  refresh_right_pane(NULL);
	  return;
     }

     tree = GTK_TREE(GTK_TREE_ITEM(tree_item)->subtree);

     if( (items = tree->children) == NULL)
	  return;

     g_node_children_foreach(node, G_TRAVERSE_ALL,
			     (GNodeForeachFunc) node_entry_destroy, NULL);

     gtk_tree_remove_items(tree, items);

     new_subtree = gtk_tree_new();
     gtk_tree_item_set_subtree(GTK_TREE_ITEM(tree_item), new_subtree);
     gtk_signal_connect(GTK_OBJECT(new_subtree), "select_child",
			GTK_SIGNAL_FUNC(tree_item_selected), NULL);

     gtk_signal_connect(GTK_OBJECT(tree_item), "expand",
			GTK_SIGNAL_FUNC(attach_children_to_node),
			NULL);

     if(was_selected)
	  refresh_right_pane(NULL);

     if(was_expanded)
	  gtk_signal_emit_by_name(GTK_OBJECT(tree_item), "expand");

}


/*
 * a server was selected in the tree widget.
 *
 * FIXME: put up the server's editable info here? Easy to do, but
 *        does it belong here?
 */
void server_item_selected(GtkWidget *treeroot, GtkWidget *tree_item)
{
     GNode *node;
     GtkWidget *pane2_vbox, *label;
     char *server_name;

     if( (node = (GNode *) gtk_object_get_data(GTK_OBJECT(tree_item), "node")) == NULL)
	  return;

     server_name = dn_by_node(node);

     if(strncmp(cur_dn->str, server_name, cur_dn->len)) {
	  g_string_assign(cur_dn, server_name);

	  pane2_vbox = gtk_object_get_data(GTK_OBJECT(pane2_scrwin), "vbox");
	  gtk_widget_destroy(pane2_vbox);

	  pane2_vbox = gtk_vbox_new(FALSE, 2);
	  gtk_object_set_data(GTK_OBJECT(pane2_scrwin), "vbox", pane2_vbox);
	  gtk_widget_show(pane2_vbox);
	  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pane2_scrwin),
						pane2_vbox);

	  label = gtk_label_new(server_name);
	  gtk_widget_show(label);
	  gtk_box_pack_start(GTK_BOX(pane2_vbox), label, FALSE, FALSE, 0);

     }

}


/*
 * display an inputbox for the selected item in the right pane
 */
void tree_item_selected(GtkWidget *treeroot, GtkWidget *tree_item)
{
     GNode *node;
     GtkWidget *pane2_vbox;
     struct ldapserver *server;
     struct node_entry *entry;
     char *dn;

     if( (node = (GNode *) gtk_object_get_data(GTK_OBJECT(tree_item), "node")) == NULL)
	  return;

     if( (server = server_by_node(node)) == NULL)
	  return;

     dn = dn_by_node(node);

     if(strncmp(cur_dn->str, dn, cur_dn->len) || server != cur_dn_server) {
	  cur_dn_server = server;
	  g_string_assign(cur_dn, dn);

	  pane2_vbox = gtk_object_get_data(GTK_OBJECT(pane2_scrwin), "vbox");
	  gtk_widget_destroy(pane2_vbox);

	  pane2_vbox = gtk_vbox_new(FALSE, 2);
	  gtk_object_set_data(GTK_OBJECT(pane2_scrwin), "vbox", pane2_vbox);
	  gtk_widget_show(pane2_vbox);
	  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pane2_scrwin),
						pane2_vbox);

	  gtk_object_set_data(GTK_OBJECT(pane2_vbox), "dn", dn);

	  set_busycursor();

	  edit_entry_table(pane2_vbox, server, dn, TRUE);

	  /* while we're at it: if this DN still has a dummy tree, scan it */
	  entry = (struct node_entry *) node->data;
	  if(entry->status == STATUS_UNKNOWN)
	       attach_children_to_node(tree_item);

	  set_normalcursor();

     }

}


/*
 * refresh entry in right pane (reload from server)
 */
void refresh_right_pane(GtkWidget *dummy)
{
     GNode *node;
     GtkWidget *pane2_vbox;
     struct ldapserver *server;

     /* if the parent tree was refreshed while closed, the current dn
	isn't actually in the node tree. Just ignore requests for a
	detail refresh for now */
     if( (node = node_by_cur_dn()) == NULL)
	  return;

     pane2_vbox = gtk_object_get_data(GTK_OBJECT(pane2_scrwin), "vbox");
     gtk_widget_destroy(pane2_vbox);

     pane2_vbox = gtk_vbox_new(FALSE, 2);
     gtk_object_set_data(GTK_OBJECT(pane2_scrwin), "vbox", pane2_vbox);
     gtk_widget_show(pane2_vbox);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pane2_scrwin),
					   pane2_vbox);

     if( (server = server_by_node(node)) == NULL)
	  return;

     gtk_object_set_data(GTK_OBJECT(pane2_vbox), "dn", dn_by_node(node));

     set_busycursor();

     edit_entry_table(pane2_vbox, server, dn_by_node(node), 1);

     set_normalcursor();

}


/*
 * clear right pane
 */
void clear_right_pane(void)
{
     GtkWidget *pane2_vbox;

     pane2_vbox = gtk_object_get_data(GTK_OBJECT(pane2_scrwin), "vbox");
     gtk_widget_destroy(pane2_vbox);

     pane2_vbox = gtk_vbox_new(FALSE, 2);
     gtk_object_set_data(GTK_OBJECT(pane2_scrwin), "vbox", pane2_vbox);
     gtk_widget_show(pane2_vbox);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pane2_scrwin),
					   pane2_vbox);

}


/*
 * figure out the GNode by cur_dn and cur_dn_server
 */
GNode *node_by_cur_dn(void)
{
     GNode *node, *server_node;

     server_node = node = NULL;

     /* find server in root */
     g_node_children_foreach(root, G_TRAVERSE_ALL,
			     (GNodeForeachFunc) set_server_callback, &server_node);

     if(server_node)
	  /* find dn under server */
	  g_node_traverse(server_node, G_IN_ORDER, G_TRAVERSE_ALL, -1,
			  (GNodeTraverseFunc) set_node_callback, &node);

     return(node);
}


void set_server_callback(GNode *node, gpointer *server)
{
     struct node_entry *entry;

     entry = node->data;

     if(server_by_name(dn_by_node(node)) == cur_dn_server)
	  *server = node;

}


gboolean set_node_callback(GNode *node, gpointer *data)
{
     struct node_entry *entry;

     entry = node->data;

     if(!strncmp(cur_dn->str, entry->dn, cur_dn->len)) {
	  *data = node;
	  return(TRUE);
     }

     return(FALSE);
}


/*
 * move up the gnode tree until root + 1 level, which is the server entry
 */
struct ldapserver *server_by_node(GNode *node)
{
     GNode *prevnode;

     prevnode = node;
     while(!G_NODE_IS_ROOT(node)) {
	  prevnode = node;
	  node = node->parent;
     }

     return(server_by_name(dn_by_node(prevnode)));
}


/*
 * returns this node's DN
 */
char *dn_by_node(GNode *node)
{
     struct node_entry *entry;

     if(node) {
	  entry = node->data;
	  if(entry && entry->dn)
	       return(entry->dn);
     }

     return("");
}


/*
 * allocate and initialize the node_entry struct which
 * lives in the node's data field
 */
struct node_entry *new_node_entry(char *dn)
{
     struct node_entry *new_entry;

     new_entry = MALLOC(sizeof(struct node_entry), "struct node_entry");
     new_entry->status = 0;
     new_entry->dn = g_strdup(dn);

     return(new_entry);
}


/*
 * get all suffixes a server considers itself authorative for.
 */
GSList *get_suffix(struct ldapserver *server)
{
     LDAP *ld;
     LDAPMessage *res, *e;
     BerElement *ptr;
     GSList *suffixes;
     int msg, i, j, s, num_suffixes;
     char *attr, **vals, message[128], suffix[MAX_SUFFIX_LEN];
     char *ldapv3_config[] = {
	  "namingcontexts",
	  NULL
     };

     num_suffixes = 0;

     set_busycursor();

     if( (ld = open_connection(server)) == NULL) {
	  set_normalcursor();
	  return(NULL);
     }

     suffixes = NULL;

     /* try LDAP V3 style config */
     statusbar_msg("base search on NULL DN");
     msg = ldap_search_s(ld, "", LDAP_SCOPE_BASE, "(objectclass=*)",
			 ldapv3_config, 0, &res);
     if(msg == LDAP_SUCCESS) {
	  e = ldap_first_entry(ld, res);
	  if(e) {
	       for(attr = ldap_first_attribute(ld, e, &ptr); attr;
		   attr = ldap_next_attribute(ld, e, ptr)) {

		    if(!strcasecmp(attr, "namingcontexts")) {
			 vals = ldap_get_values(ld, e, attr);
			 for(i = 0; vals[i]; i++) {
			      suffixes = g_slist_append(suffixes, g_strdup(vals[i]));
			      num_suffixes++;
			 }
		    }
	       }
	  }
     }

     if(num_suffixes == 0) {

	  /* try Umich style config */
	  statusbar_msg("base search on cn=config");
	  msg = ldap_search_s(ld, "cn=config", LDAP_SCOPE_BASE,
			      "(objectclass=*)", NULL, 0, &res);

	  if(msg == LDAP_SUCCESS) {
	       e = ldap_first_entry(ld, res);
	       if(e) {
		    for(attr = ldap_first_attribute(ld, e, &ptr); attr;
			attr = ldap_next_attribute(ld, e, ptr)) {
			 if(!strcasecmp(attr, "database")) {
			      vals = ldap_get_values(ld, e, attr);
			      i = s = 0;
			      /* skip first value (backend name) */
			      while(vals[0][i] != ':' && vals[0][i])
				   i++;

			      while(vals[0][i]) {
				   while( (vals[0][i] == ':' || vals[0][i] == ' ')
					  && vals[0][i])
					i++;
				   j = 0;
				   while(vals[0][i] && vals[0][i] != ':')
					suffix[j++] = vals[0][i++];

				   /* delete trailing spaces */
				   while(suffix[j - 1] == ' ')
					j--;
				   suffix[j] = 0;

				   /* four11 and whowhere used to return nothing, recently
				      started to return empty suffixes on 'cn=config'.
				      Can't wait to see what they'll come up with next */
				   if(strlen(suffix)) {
					suffixes = g_slist_append(suffixes, g_strdup(suffix));
					num_suffixes++;
				   }
			      }
			 }
		    }
	       }
	  }

     }

     set_normalcursor();
     close_connection(server, FALSE);

     make_message(message, num_suffixes, "suffix", "suffixes", "found");
     statusbar_msg(message);

     return(suffixes);
}


void cleanup_browse_mode(void)
{

     free_browse_all();

     if(cur_dn)
	  g_string_free(cur_dn, TRUE);

}


/*
 * free() all data in the GNode tree, and destroy the tree
 */
void free_browse_all(void)
{

     if(root) {
	  /* free all node_entry structs in gnode tree */
	  g_node_traverse(root, G_LEVEL_ORDER, G_TRAVERSE_ALL, -1,
			  (GNodeTraverseFunc) node_entry_free_callback, NULL);

	  g_node_destroy(root);
	  root = NULL;
     }

}


/*
 * this does the real work for free_browse_all()
 */
gboolean node_entry_free_callback(GNode *node, gpointer dummy)
{
     struct node_entry *entry;

     if(node && node->data) {
	  entry = node->data;
	  if(entry->dn)
	       free(entry->dn);
	  FREE(entry, "struct node_entry");
	  node->data = NULL;
     }

     return(FALSE);
}


/*
 * free() all data in the node, and destroy the node for good measure
 */
void node_entry_destroy(GNode *node, gpointer dummy)
{

     node_entry_free_callback(node, dummy);
     g_node_destroy(node);

}


